# -- trae las constantes del archivo "constants.py"
from constants import *

def read_col_names(path):
    """
        Recibe el path donde esta la galaxia, ej: "Data/rcugc7876.txt". 
        El formato del archivo que se lee es: 
        primera fila: nombre de las columnas. ej: r(kpc), v(km/s), etc
        segunda fila y hasta abajo: datos.
        
        La función lee desde la segunda fila (los datos numéricos), y aparte lee la primera fila, 
        donde estan los nombres de las columnas, le quita los espacios a estos nombres.
        ej: nombre original "r  (kpc)" --> nombre posterior "r(kpc)".
        
        
        Retorna los datos como un dataframe.
    """
 
    # -- read the first row, that contains the name of each column, but with spaces
    header_raw = pd.read_csv(path,nrows=1,header=None)
    # -- convert the list with all names, into a numpy array
    header_raw = header_raw.iloc[0].to_numpy()
    # -- split the str array, if a double space appears
    h = np.array(header_raw[0].split("  "))
    # -- extract unique names and the index of these uniques in the original array
    header, index = np.unique(h,return_index=True)
    # -- extract the unique header, in the correct order 
    header = h[sorted(index)]
    # -- not consider space a name column
    header = header[1:]
    # -- convert the names into dataframe
    dfh = pd.DataFrame(list(header), columns = ["header"])
    # -- delete space inside each name
    dfh["header"] = dfh["header"].str.replace(" ","")
    columns = dfh["header"].to_numpy()
    
    return columns

def vnfw (MM, c, x, G = G, H0 = H0):
    '''
        Velocity for NFW model. 
        Recibe como parametros el M_200 y la concentracion c. G y H0 son valores dados y fijos.
        Recibe la variables x como x = r/r200, el r200 depende de M_200 y se define abajo como 
        otra función.
        
        La M200 debe estar en unidades de masa solar.
        x = r/r200 donde r y r200 deben estar en unidades parsec (pc)
        La concentración no tiene unidades
        
        Retorna el valor de la velocidad en km/s para los parámetros de M200 y c dados.
    '''
    v = (10*G*H0*MM)**(1/3) * ((np.log(1+c*x) - c*x/(1+c*x)) /(x*(np.log(1+c) - c/(1+c))) )**(1/2)
    return v

def viso (rr, x, G = G):
    '''
        Velocity for ISO model. 
        Recibe como parámetros el rho0 y Rc. El "rr" dentro de "v = ..." es rr = (rho0)(Rc^2).
        Recibe la variable x como x = r/Rc.
        
        rho0 esta dado en unidades de Msun / (pc^3)
        x = r/Rc donde r y Rc esta dado en parsec.
 
        
        Retorna el valor de la velocidad en km/s para los parámetros de rho0 y Rc dados.
    '''
    v = ( 4*np.pi*G*rr*x**(-1) *(x - np.arctan(x) ) )**(1/2)
    return v

def funcline(params,x,i = 0):
    """
        Recibe los parametros en forma de lista, array o tupla, para cada modelo (NFW o ISO), y valores del eje x
        i = 0: si es modelo NFW
        i = 1: si es modelo ISO
        
        Por default esta NFW.
        
        Retorna la función del modelo (NFW o ISO) al cual se le quiere hacer ajuste por
        least square.
    """
    if i == 0:
        r200 = ((G * params[0] / (10*G*H0*params[0])**(2/3))*units.pc).value
        function = vnfw(params[0], params[1], x/r200, G = G, H0 = H0)
    else:
        function = viso (params[0]*(params[1]**2), x/params[1], G = G)
    return function

def Errorlineal(tpl,x,y,incer=1,i=0):
    """
        Recibe los parámetros (tpl), los valores de x, para calcular la velocidad "teórica" de acuerdo a los parámetros
        y los valores de la velocidad observada (y).
        Recibe el valor de la incertidumbre (en este caso dado por la columna "err_v(km/s)"),
        por defaul esta en 1 por si no queremos considerar o no hay errores.
        Recibe i = 0 (NFW) o i = 1 (ISO)
    """
    return (funcline(tpl,x,i)-y)/ (np.sqrt(abs(incer))**2)

def calculator(galaxy,p0,Errorlineal,incer=1,i=0):
    """
        Recibe la dataframe de los datos de la galaxia (galaxy)
        Recibe los parametros (p0) de partida para least square
        Recibe la función de Errorlineal.
        Recibe el valor de la incertidumbre (en este caso dado por la columna "err_v(km/s)"),
        por defaul esta en 1 por si no queremos considerar o no hay errores.
        Recibe i = 0 (NFW) o i = 1 (ISO)
        
        Retorna el mejor fit a los datos observados mediante la función de least square.
        Retorna la "matriz de covarianza" **no es exactamente esa matriz
    """
    
    x = galaxy["r(kpc)"].to_numpy()*1000
    best,covx, infodict, errmsg, success= leastsq(Errorlineal, p0, 
                                                  args=(x,galaxy["v(km/s)"].to_numpy(),incer,i),
                                                 full_output=1, epsfcn=0.0001)
    return best,x,covx


def uncertainty(best,x,covx, galaxy, p0,i=0):
    
    """
    [1](https://docs.scipy.org/doc/scipy/reference/generated/scipy.optimize.leastsq.html)
    To obtain the covariance matrix of the parameters x, cov_x must be multiplied by the 
    variance of the residuals .
   
    
    [2](https://docs.scipy.org/doc/scipy/reference/generated/scipy.optimize.curve_fit.html) 
    The estimated covariance of popt. The diagonals provide the variance of the parameter
    estimate. To compute one standard deviation errors on the parameters use 
    perr = np.sqrt(np.diag(pcov)).
    
    Recibe el mejor fit a los datos, la matriz de covarianza, el dataframe de la galaxia a trabajar, 
    los parámetros inciales y i = 0 (NFW) o i = 1 (ISO) 
    
    Retorna una lista del mejor parámetro y su respectiva incertidumbre formal
    ej: M200 = (10e10, 1.0e9)
    
    """
    
    if i == 0:
        residual_err = (Errorlineal(best,x,galaxy["v(km/s)"].to_numpy(),
                           incer=galaxy["err_v(km/s)"].to_numpy(),i=0)**2).sum() / (len(galaxy["v(km/s)"]) - len(p0))
    else:
        residual_err = (Errorlineal(best,x,galaxy["v(km/s)"].to_numpy(),
                           incer=galaxy["err_v(km/s)"].to_numpy(),i=1)**2).sum() / (len(galaxy["v(km/s)"]) - len(p0))
    
    covm = covx*residual_err
    err = np.sqrt(np.diag(abs(covm)))
    
    return list(zip(best,err))

def determine(gal,label):
    """
        Recibe el dataframe de la galaxia a trabajar, puede ser considerando todos 
        los puntos RA (receiding - approaching) o solo R o solo A y
        el label = ["ra","r","a"]. 
        
        Hace el fit para modelo NFW ( i = 0) y para ISO ( i = 1) y para cada uno de 
        estos, considera es "ra", "r" o "a" 
       
        Retorna una lista indicando si se hizo para NFW (0) o ISO (1), el mejor parámetro, 
        su respectiva incertidumbre formal y si es "ra", "r" o "a"
        
        ej: Modelo NFW, parametro M200 
        = ("0", 10e10, 1.0e9, "ra",),("0", 11e10, 2.0e9,"r"),("0", 10.5e10, 1.3e9,"a")
    """
    AA = []
    for ii in [0,1]:
        if ii == 0:
            p0 = [M200,c]
        else:
            p0 = [rho0,Rc]
            
        best,x,covx = calculator(gal, p0, Errorlineal, gal["err_v(km/s)"].to_numpy(),ii)
        try: 
            unce = uncertainty(best,x,covx,gal,p0,ii)
        except:
            unce = list(zip(best,np.zeros(len(best))))
            pass
        
    
        AA.append((ii,unce,label))
    return AA

def return_values(AA):
    """
        Recibe la lista AA dada por la función determine(gal,label) y la desgloza para construir un data frame
        donde se reporte el parámetro del mejor fit  con su respectivo error formal y para cada modelo (NFW o ISO)
        
        Retorna dataframe con el 
        * nombre de la galaxia,
        * side: "ra", "r" o "a"
        * M200_NFW(Msun)
        * err_M200_NFW(Msun)
        * c_NFW
        * err_c_NFW
        * rho0_ISO(Msun/pc3)
        * err_rho0_ISO(Msun/pc3)
        * Rc_ISO(pc)
        * err_Rc_ISO(pc)
    """
    
    A = np.array(AA)
    ID = A[:,0]
    RA =[i[0][-1] for i in A[:,1]]
    M200_NFW = [i[0][1][0][0] for i in A[:,1]]
    err_M200_NFW  = [i[0][1][0][1] for i in A[:,1]]
    c_NFW = [i[0][1][1][0] for i in A[:,1]]
    err_c_NFW = [i[0][1][1][1] for i in A[:,1]]
    rho0_ISO = [i[1][1][0][0] for i in A[:,1]]
    err_rho0_ISO = [i[1][1][0][1] for i in A[:,1]]
    Rc_ISO = [i[1][1][1][0] for i in A[:,1]]
    err_Rc_ISO = [i[1][1][1][1] for i in A[:,1]]
    
    datas = pd.DataFrame({"ID":ID,"Side":RA,
                      "M200_NFW(Msun)":M200_NFW,"err_M200_NFW(Msun)":err_M200_NFW,
                      "c_NFW":c_NFW,"err_c_NFW":err_c_NFW,
                      "rho0_ISO(Msun/pc3)":rho0_ISO,"err_rho0_ISO(Msun/pc3)":err_rho0_ISO,
                      "Rc_ISO(pc)":Rc_ISO,"err_Rc_ISO(pc)":err_Rc_ISO})
    
    return datas

# -- esta funcioncita cálcula el r200 de acuerdo a la ecuación 1.14 de la tesis, donde M200 (pm) viene
# -- dado por la tabla de datos del paper (M_halo) y V200 viene dados por la ecuación 7 del paper 
# -- y pie de nota en página 4 del paper
r200 = lambda pm : ((G * pm / (10*G*H0*pm)**(2/3))*units.pc).value









# -------- chi2
def grafica_data_ybestfit(path, gal, df_best_parmeters, ss="ra"):
    '''
        Entradas:
            path: path de los datos de la galaxia
            
            gal: dataframe de la galaxia a trabajar, dependiendo de "ra", "r" ó "a"
            
            df_best_parmeters: dataframe que tiene los mejores parametros para NFW e ISO y para "ra", "r" y "a", es
            el output de a función return_values(AA)
            
            ss: Side --> "ra", "r" ó "a"
            
            mtype: "leastsq": toma parametros dados por leastsq
                   "new"    : toma parametros dados por el chi^2, luego de exploración
                   
            new_para_min_ch2_b: 0: no se ha hecho aún exploración, no hay nuevo chi^2
                              : arreglo de longitud 4, los dos primeros son los parametros (M200,c) para el nuevo 
                              chi^2 para NFW y los dos ultimos para ISO
                              
        Salidas:
            Gráfica de los datos y las curvas NFW e ISO
    '''
    
    
    if ss == "ra":
        j = 0
    if ss == "r":
        j = 1
    if ss == "a":
        j = 2   
    # -- gráfica de los datos 
    
    grilla = gridspec.GridSpec(3, 3)
    fig = plt.figure(figsize = (11,8))

    ax1 = plt.subplot(grilla[0:2,:])
    # --  quitar ticks
    plt.xticks([])
    x = np.linspace(0.02,max(gal["r(kpc)"])*1000,1000)
    ax1.errorbar(gal["r(kpc)"], gal["v(km/s)"], yerr=gal["err_v(km/s)"],xerr=gal["err_r(kpc)"],
                fmt='o', markersize='2',
                color = "purple", ecolor="black",capsize=2,elinewidth=0.5)
    
    # -- función para valor de y o velocidad para nfw e iso, dado la distancia xx
    ynfw = lambda xx: vnfw(df_best_parmeters["M200_NFW(Msun)"][j], df_best_parmeters["c_NFW"][j], xx/r200(df_best_parmeters["M200_NFW(Msun)"][j]), G = G, H0 = H0)

    yiso = lambda xx: viso ((df_best_parmeters["rho0_ISO(Msun/pc3)"][j]*df_best_parmeters["Rc_ISO(pc)"][j]**2), xx/abs(df_best_parmeters["Rc_ISO(pc)"][j]), G = G)

    # -- gráficar el ajuste para nfw e iso
    ax1.plot(x/1000,ynfw(x),
            color="darkorange", label = r"NFW"
            "\n" r"M$_{200}$ = %.2e" %df_best_parmeters["M200_NFW(Msun)"][j]+r"M$_{\odot}$"
            "\n" "c = %.2f" %df_best_parmeters["c_NFW"][j])
            #label="NFW\nM200 = {:.2e}Msun\nc = {:.2f}".format(df_best_parmeters["M200_NFW(Msun)"][j],df_best_parmeters["c_NFW"][j]))

    ax1.plot(x/1000,yiso(x),
            color="blue", label= r"ISO"
            "\n" r"$\rho_0$ = %.2f" %abs(df_best_parmeters["rho0_ISO(Msun/pc3)"][j])+r"(M$_{\odot}$/pc$^3$)"
            "\n" r"R$_c$ = %.2f" %abs(df_best_parmeters["Rc_ISO(pc)"][j])+"pc")


    # -- calcular las velocidad al sumar o restar los errores dados por leastsq para cada parametro
    yup = viso ((df_best_parmeters["rho0_ISO(Msun/pc3)"][j]+df_best_parmeters["err_rho0_ISO(Msun/pc3)"][j])*(df_best_parmeters["Rc_ISO(pc)"][j]+df_best_parmeters["err_Rc_ISO(pc)"][j])**2, x/(abs(df_best_parmeters["Rc_ISO(pc)"][j])+df_best_parmeters["err_Rc_ISO(pc)"][j]), G = G)
    ydow = viso ((df_best_parmeters["rho0_ISO(Msun/pc3)"][j]-df_best_parmeters["err_rho0_ISO(Msun/pc3)"][j])*(df_best_parmeters["Rc_ISO(pc)"][j]-df_best_parmeters["err_Rc_ISO(pc)"][j])**2, x/(abs(df_best_parmeters["Rc_ISO(pc)"][j])-df_best_parmeters["err_Rc_ISO(pc)"][j]), G = G)

    # -- colorear la región de value+error y value-error
    ax1.fill_between(x/1000, yup, ydow, alpha=0.3)

    # -- lo mismo pero para nfw
    yup = vnfw((df_best_parmeters["M200_NFW(Msun)"][j]+df_best_parmeters["err_M200_NFW(Msun)"][j]), (df_best_parmeters["c_NFW"][j]+df_best_parmeters["err_c_NFW"][j]), x/r200(df_best_parmeters["M200_NFW(Msun)"][j]+df_best_parmeters["err_M200_NFW(Msun)"][j]), G = G, H0 = H0)
    ydow = vnfw((df_best_parmeters["M200_NFW(Msun)"][j]-df_best_parmeters["err_M200_NFW(Msun)"][j]), (df_best_parmeters["c_NFW"][j]-df_best_parmeters["err_c_NFW"][j]), x/r200(df_best_parmeters["M200_NFW(Msun)"][j]-df_best_parmeters["err_M200_NFW(Msun)"][j]), G = G, H0 = H0)

    ax1.fill_between(x/1000, yup, ydow, alpha=0.3)

    ax2 = plt.subplot(grilla[2:,:])
    # -- graficar residuo
    ax2.plot(gal["r(kpc)"],(ynfw(gal["r(kpc)"]*1000) - gal["v(km/s)"]) / (np.sqrt(gal["err_v(km/s)"])**2) ,'--r',color="darkorange")
    ax2.plot(gal["r(kpc)"],(yiso(gal["r(kpc)"]*1000) - gal["v(km/s)"]) / (np.sqrt(gal["err_v(km/s)"])**2) ,'--r',color="blue")

        
    # -- poner nombre a ejes y título a la gráfica
    ax1.set_title(path.split("/")[-1][:-4])
    ax2.set_xlabel(gal["r(kpc)"].name)
    ax1.set_ylabel(gal["v(km/s)"].name)
    ax2.set_ylabel("residuo(km/s)")
    
    # -- poner el cuadrito de los labels siempre en el costado inferior derecho
    ax1.legend(loc="lower right")
    #plt.show()
    
    
    # -- guardar gráficas

    plt.savefig("../Results/{}RA.pdf".format(path.split("/")[-1][:-4]))
    
    
    
def parameters_range(df_best_parmeters, i = 0, ss = "ra"):
    '''
        Entradas:
            df_best_parmeters: dataframe que tiene los mejores parametros para NFW e ISO y para "ra", "r" y "a", es
            el output de a función return_values(AA)
            
            i = 0: Modelo a usar, NFW --> i = 0, ISO --> i = 1
            
            ss: Side --> "ra", "r" ó "a"
            
        Salidas:
            Lista compuesta por tuplas, cada tupla contiene todas las posibles combinaciones entre los parámetros.
    '''
    
        
    if ss == "ra":
        j = 0
    if ss == "r":
        j = 1
    if ss == "a":
        j = 2
        
        
    # -- extrae orden de magnitud de cada parametro
    order_M200 = math.floor(math.log10(df_best_parmeters["M200_NFW(Msun)"][j]))
    order_c = math.floor(math.log10(df_best_parmeters["c_NFW"][j]))
    order_Rc = math.floor(math.log10(abs(df_best_parmeters["Rc_ISO(pc)"][j])))
    order_rho = math.floor(math.log10(df_best_parmeters["rho0_ISO(Msun/pc3)"][j]))

    # -- crea un array de 300 valores entre un valor min y un max, para cada parametros. 
    # -- Cada modelo va a tener una matriz de 300x300 que corresponde a las combinaciones de los parametros
    
    # -- si el modelo que se esta trabajando es el NFW        
    if i == 0:        
        c_range = np.linspace(((df_best_parmeters["c_NFW"][j]/(10**order_c)) - ((df_best_parmeters["c_NFW"][j]/(10**order_c))-0.5))*(10**order_c),
                              ((df_best_parmeters["c_NFW"][j]/(10**order_c)) + 4)*(10**order_c),
                              300)
        M200_range = np.linspace(((df_best_parmeters["M200_NFW(Msun)"][j]/(10**order_M200)) - ((df_best_parmeters["M200_NFW(Msun)"][j]/(10**order_M200))-0.5))*(10**order_M200),
                              ((df_best_parmeters["M200_NFW(Msun)"][j]/(10**order_M200)) + 5)*(10**order_M200),
                              300)
        
        return [(M200,c) for M200 in M200_range for c in c_range]
    
    # -- si el modelo que se esta trabajando es el ISO
    else:     
        Rc_range = np.linspace(((abs(df_best_parmeters["Rc_ISO(pc)"][j])/(10**order_Rc)) - ((abs(df_best_parmeters["Rc_ISO(pc)"][j])/(10**order_Rc))-0.5))*(10**order_Rc),
                              ((abs(df_best_parmeters["Rc_ISO(pc)"][j])/(10**order_Rc)) + 4)*(10**order_Rc),
                              300)
        
        rho0_range = np.linspace(((abs(df_best_parmeters["rho0_ISO(Msun/pc3)"][j])/(10**order_rho)) - ((abs(df_best_parmeters["rho0_ISO(Msun/pc3)"][j])/(10**order_rho))-0.5))*(10**order_rho),
                              ((abs(df_best_parmeters["rho0_ISO(Msun/pc3)"][j])/(10**order_rho)) + 4)*(10**order_rho),
                              300)
        
        return [(Rc,rho0) for Rc in Rc_range for rho0 in rho0_range]
    
    
    
def chi2_parameters_range(gal, combs, i = 0):
    '''
        Entradas:
            gal: dataframe de la galaxia a trabajar, dependiendo de "ra", "r" ó "a"
            
            combs: lista de tuplas con todas las combinaciones de parámetros, es el output de la función
            parameters_range(df_best_parmeters, i = 0, ss = "ra")
            
            i = 0: Modelo a usar, NFW --> i = 0, ISO --> i = 1
            
        Salidas:
            Lista con el chi^2 para cada tupla de parametros.
            
    '''
    chis = []
    # -- recorre toda la matriz de parámetros y calcula el chi^2
    for ii in range(len(combs)):
        # -- NFW: para1 = M200
        #         para2 = c
        # -- ISO: para1 = R_c
        #         para2 = rho0
        
        para1 = combs[ii][0]
        para2 = combs[ii][1]
        # -- si es modelo NFW
        if i == 0:
            y_pred = vnfw(para1, para2, gal["r(kpc)"]*1000/r200(para1), G = G, H0 = H0)
        # -- si es modelo ISO
        else:
            y_pred = viso ( (para2*para1**2) , gal["r(kpc)"]*1000/abs(para1), G = G)
        
        # -- calcula los chi^2
        chi2n = (((y_pred-gal["v(km/s)"])/(gal["err_v(km/s)"]**(1/1)))**2).sum()
        chis.append((chi2n/(len(gal)-2)))
        
    return chis






def chi2_intervals(df_best_parmeters, gal, chis, i = 0, ss = "ra"):
    '''
        Entradas:
            df_best_parmeters: dataframe que tiene los mejores parametros para NFW e ISO y para "ra", "r" y "a", es
            el output de a función return_values(AA)
            
            gal: dataframe de la galaxia a trabajar, dependiendo de "ra", "r" ó "a"
            
            chis: Lista con el chi^2 para cada tupla de parametros, es el output de la función 
            chi2_parameters_range(gal, combs, i = 0)
            
            i = 0: Modelo a usar, NFW --> i = 0, ISO --> i = 1
            
            ss: Side --> "ra", "r" ó "a"
            
        Salidas:
            Lista de tuplas, donde cada tupla (index, chi2), el index que da razón de a que par de parámetros
            corresponde (en la lista de combs) el chi2
            
            Retorna 5 listas:
            ch2_m: chi2 <= chi2_bestfit
            ch2_p01: chi2_bestfit<ch2<chi2_bestfit+0.006
            ch2_p12: chi2_bestfit+0.006<ch2<=chi2_bestfit+0.093
            ch2_p23: chi2_bestfit+0.093<ch2<=chi2_bestfit+0.765
            ch2_p3p: chi2_bestfit+0.765<ch2
            
    '''
    
    if ss == "ra":
        j = 0
    if ss == "r":
        j = 1
    if ss == "a":
        j = 2
        
    # -- si es modelo NFW
    if i == 0:
        # -- calcula las velocidades para los parametros dados por leastsq
        y_pred_bestfit = vnfw(df_best_parmeters["M200_NFW(Msun)"][j], df_best_parmeters["c_NFW"][j], 
                              gal["r(kpc)"]*1000/r200(df_best_parmeters["M200_NFW(Msun)"][j]), G = G, H0 = H0)
    # -- si es modelo ISO
    else:
        # -- calcula las velocidades para los parametros dados por leastsq
        y_pred_bestfit = viso((df_best_parmeters["rho0_ISO(Msun/pc3)"][j]*df_best_parmeters["Rc_ISO(pc)"][j]**2), 
                              gal["r(kpc)"]*1000/abs(df_best_parmeters["Rc_ISO(pc)"][j]), G = G)
    
    # -- calcula el chi2 para esos valores, el chi2 "original"
    chi2_bestfit = ((((y_pred_bestfit-gal["v(km/s)"])/(gal["err_v(km/s)"]))**2).sum())/(len(gal)-2)
    
    
    # -- mira que chi2 son menores que el obtenido por leastsq
    ch2_m = [(i,ch2) for i,ch2 in enumerate(chis) if ch2<=chi2_bestfit]
    # -- extrae el nuevo chi minimo
    #i_chi2_bestfit = [i for i in ch2_m if i[1] == min(list(zip(*ch2_m))[1])][0][0]
    # -- poner ese nuevo minimo como chi de referencia
    #chi2_bestfit = chis[i_chi2_bestfit]
    
    ch2_m = [(i,ch2) for i,ch2 in enumerate(chis) if ch2<=chi2_bestfit]
    ch2_p01 = [(i,ch2) for i,ch2 in enumerate(chis) if chi2_bestfit<ch2<=chi2_bestfit+0.006]
    ch2_p12 = [(i,ch2) for i,ch2 in enumerate(chis) if chi2_bestfit+0.006<ch2<=chi2_bestfit+0.093]
    ch2_p23 = [(i,ch2) for i,ch2 in enumerate(chis) if chi2_bestfit+0.093<ch2<=chi2_bestfit+0.765]
    ch2_p3p = [(i,ch2) for i,ch2 in enumerate(chis) if chi2_bestfit+0.765<ch2]
    
#     if len(ch2_m) == 0:
#         ch2_m = (i,chi2_bestfit)
#     if len(ch2_p01) == 0:
#         ch2_p01 = (i,chi2_bestfit)
    
    return ch2_m,ch2_p01,ch2_p12,ch2_p23,ch2_p3p


def parameters_each_interval(combs, ch2_inter, kk = 0):
    '''
        Entradas:
            combs: lista de tuplas con todas las combinaciones de parámetros (para1, para2), es el output de la función
            parameters_range(df_best_parmeters, i = 0, ss = "ra")
            ch2_inter: Lista de tuplas, donde cada tupla (index, chi2), el index que da razón de a que par de parámetros
            corresponde (en la lista de combs) el chi2, es el output de la función 
            chi2_intervals(df_best_parmeters, gal, chis, i = 0, ss = "ra")
    
                ch2_m: chi2 < = chi2_bestfit --> kk = 0
                ch2_p01: chi2_bestfit<ch2<chi2_bestfit+0.006 --> kk = 1
                ch2_p12: chi2_bestfit+0.006<ch2<=chi2_bestfit+0.093 --> kk = 2
                ch2_p23: chi2_bestfit+0.093<ch2<=chi2_bestfit+0.765 --> kk = 3
                ch2_p3p: chi2_bestfit+0.765<ch2 --> kk = 4
            
            kk = 0: Que intervalo se quiere de acuerdo a las 5 listas
            
        Salidas:
            Valores de parametro 1 y parametro 2 que cumplen estar dentro de un intervalo de chi^2
    
    '''

    # -- kk indica que región de chi^2 se quiere. De acuerdo a que región se quiere,
    # -- se separa la tupla de parámetros, para que cada uno quede separadao y graficar
    
    combs_interval = np.array(combs)[np.array(list(zip(*ch2_inter[kk]))[0])]
    para1_interval = list(zip(*combs_interval))[0]
    para2_interval = list(zip(*combs_interval))[1]

    
    
    return para1_interval,para2_interval




def plot_ellipse_confidence_interval(path, para_range, df_best_parmeters, ss="ra", i = 0):
    '''
        Entradas:
            path: path de los datos de la galaxia
            
            para_range: arreglo de longitud 5, donde se ponen el par de parámetros que cumplen:
                chi2 < =chi2_bestfit --> kk = 0
                chi2_bestfit<ch2<chi2_bestfit+0.006 --> kk = 1
                chi2_bestfit+0.006<ch2<=chi2_bestfit+0.093 --> kk = 2
                chi2_bestfit+0.093<ch2<=chi2_bestfit+0.765 --> kk = 3
                chi2_bestfit+0.765<ch2 --> kk = 4
            es el output de la función parameters_each_interval(combs, ch2_inter, kk = 0) 
            luego de realizar un for para todos los kk (por eso queda de longitud 5)
            
            df_best_parmeters: dataframe que tiene los mejores parametros para NFW e ISO y para "ra", "r" y "a", es
            el output de a función return_values(AA)
            
            new_para_min_ch2: parametros que tiene ahora el menor chi^2 luego de la exploración para NFW e ISO
            
            i = 0: Modelo a usar, NFW --> i = 0, ISO --> i = 1
            
            ss: Side --> "ra", "r" ó "a"
            
        Salidas:
            Gráfica de las elipses de intervalos de confianza, de acuerdo al incremento en chi^2, los ejes
            son los parámetros del modelo.
            
    '''
    if ss == "ra":
        j = 0
    if ss == "r":
        j = 1
    if ss == "a":
        j = 2
        
    # --  gráfica el par de parámetros que estand entro de cada región de chi^2 con colores diferentes    
    plt.figure(figsize = (10,7))
#     plt.scatter(para_range[0][0],para_range[0][1],color="blue",alpha=0.5,label = r"$\chi^2\leq\chi^2_{fit}$")
    plt.scatter(para_range[0][0],para_range[0][1],color="red",alpha=0.5,label = r"$\chi^2_{fit}<\chi^2\leq\chi^2_{fit}+0.006$")
    plt.scatter(para_range[1][0],para_range[1][1],color="green",alpha=0.5,label = r"$\chi^2_{fit}+0.006<\chi^2\leq\chi^2_{fit}+0.093$")
    plt.scatter(para_range[2][0],para_range[2][1],color="violet",alpha=0.5,label = r"$\chi^2_{fit}+0.093<\chi^2\leq\chi^2_{fit}+0.765$")
    #plt.scatter(para_range_NFW_ra_p3p[0],para_range_NFW_ra_p3p[1],color="orange",alpha=0.5,label = r"$\chi^2_{fit}+3<\chi^2$")

    # -- si el modelo es NFW
    if i == 0:
        plt.scatter(df_best_parmeters["M200_NFW(Msun)"][j],df_best_parmeters["c_NFW"][j],color="black")
#         plt.scatter(new_para_min_ch2[0],new_para_min_ch2[1], color = "orange",label="New min $\chi^2$ " )
        plt.xlabel("M_200(Msun)")
        plt.ylabel("c")
        plt.title("{} NFW".format(path.split("/")[-1][:-4]))
    
    # -- si el modelo es ISO
    else:
        plt.scatter(abs(df_best_parmeters["Rc_ISO(pc)"][j]),df_best_parmeters["rho0_ISO(Msun/pc3)"][j],color="black")
#         plt.scatter(new_para_min_ch2[0],new_para_min_ch2[1], color = "orange",label="New min $\chi^2$ " )
        plt.xlabel("Rc_ISO(pc)")
        plt.ylabel("rho0_ISO(Msun/pc3)")
        plt.title("{} ISO".format(path.split("/")[-1][:-4]))
    # plt.ylim(0,30)
    # plt.xlim(3e10,7e10)
    plt.legend()
    
    # -- guarda la image, el nombre cambia si es NFW o ISO
    plt.savefig("../Results/{}RA{}.pdf".format(path.split("/")[-1][:-4], i))

